/* ==========================================================================
  
        FFFFF  RRRR   EEEEE  EEEEE  FFFFF   OOOO   RRRR   M    M 
        F      R   R  E      E      F      O    O  R   R  MM  MM 
        FFF    RRRR   EEE    EEE    FFF    O    O  RRRR   M MM M 
        F      R  R   E      E      F      O    O  R  R   M    M 
        F      R   R  E      E      F      O    O  R   R  M    M 
        F      R   R  EEEEE  EEEEE  F       OOOO   R   R  M    M 
  
  
             SSSSS  PPPPP   L      III  N    N  EEEEE   SSSSS    
            S       P    P  L       I   NN   N  E      S
             SSSS   PPPP    L       I   N N  N  EEE     SSSS
                 S  P       L       I   N  N N  E           S
                 S  P       L       I   N   NN  E           S
            SSSSS   P       LLLLL  III  N    N  EEEEE  SSSSS 
  
   ==========================================================================
    
     Project:      "Freeform splines software package" of 
  
                       University of Karlsruhe (TH)
                       Institute for Operating and Dialog Systems
                       Research Group on Computer Aided Geometric Design
                       Head of research group: Prof. Dr. H. Prautzsch
                       WWW: http:  i33www.ira.uka.de
  
  
     Program name: bez2bezmesh
  
     Version:      1.1
  
     Author:       Stephan Bischoff
    
     Date:         September, 1999
  
     Usage:        bez2bezmesh BEZ-FILE MESH-FILE
  
     Description:  The program reads BEZ-FILE and writes out the corresponding
                   Bezier nets to MESH-FILE. It can handle patches up to 
		   bidegree (15,15). Patches of bidegree > (9,9) are given in
		   hexadecimal notation, e.g.
		         BEZDD3
		   means a patch of bidegree (13,13).

     Contact:      Please let use hear from you in case you have any 
                   suggestions or problems with our software:
                          prau@ira.uka.de
                          umlauf@ira.uka.de
*/

#include <math.h>
#include <iostream.h>
#include <stdlib.h>
#include <ctype.h>

#include "scanner.c"   /* generated by flex */
  

/* fehler : writes an error message and aborts the programme */
void fehler(char *file_name, char *message)
{
  fprintf(stderr, "%s : %d : %s\n", file_name, line_no, message);
  fclose(outfile);
  remove(file_name);
  exit(1);
}

/* get_numbers : tries to read n numbers */
int get_numbers(int n)
{
  for ( ; n > 0; n--) 
    if ((tok = yylex()) != TOK_NUMBER) return 0;
  return 1;
}

/*
 *         main
 */

int main(int argc, char **argv)
{
  if (argc != 3) {
      fprintf(stderr, "Usage : %s BEZ-FILE MESH-FILE\n", argv[0]);
      fprintf(stderr, "Reads the BEZ-FILE and extracts the Bezier-nets and\n");
      fprintf(stderr, "writes them to MESH-FILE\n");
      fprintf(stderr, "For details refer to the documentation in user_manual.ps.\n");
      return 1;
    }
  
  int i, j, k, patch = 1;

  /* open infile */
  if ((yyin = fopen(argv[1], "r")) == NULL) {
    printf("%s : can't open %s for reading\n", argv[0], argv[1]);
    return 1;
  }

  /* open outfile */
  if ((outfile = fopen(argv[2], "w")) == NULL) {
    printf("%s : can't open %s for writing\n", argv[0], argv[2]);
    fclose(yyin);
    return 1;
  }

  /* write info-header */
  fprintf(outfile, "# This is the Bezier net of %s\n", argv[1]);
  fprintf(outfile, "# Generated by the command : ");
  for (i = 0; i < argc - 1; i++)
    fprintf(outfile, "%s ", argv[i]);
  fprintf(outfile, "%s\n", argv[argc - 1]);
  fprintf(outfile, "{ LIST\n");

  /* main loop */
  tok = yylex();
  while (tok == TOK_BEZ) {

    tok = TOK_NUMBER;
    while (tok == TOK_NUMBER) {

      /* write MESH-header */
      fprintf(outfile, "# patch %d\n", patch++);
      fprintf(outfile, "{ %sMESH\n", (dimension == 3 ? "" : "4"));
      fprintf(outfile, "%d %d\n", degree_x + 1, degree_y + 1);
      
      /* read Bezier points  and write them as MESH-points */
      for (i = 0; i < degree_x + 1; i++) {
	for (j = 0; j < degree_y + 1; j++) {
	  for (k = 0; k < 3; k++) {
	    if ((tok = yylex()) == TOK_NUMBER)
	      fprintf(outfile, "%s%s", yytext, (k < 2 ? " " : (j < degree_y ? "\n" : "\n\n")));
	    else 
	      fehler(argv[2], "number expected (Bezier component)");	      
	  }
	}
      }
      
      /* throw away the texture and colour information */
      if (texture && !get_numbers(16)) fehler(argv[2], "number expected (texture component)");           
      if (colour  && !get_numbers(16)) fehler(argv[2], "number expected (colour component)");

      fprintf(outfile, "}\n");
      if ((tok = yylex())== TOK_NUMBER) yyless(0);
    }
  }
  fprintf(outfile, "}");
  
  if (tok != TOK_EOF) 
    fehler(argv[2], "syntax error");
  
  /* no errors occured */
  fclose(yyin);
  fclose(outfile);
  return 0;
}



